@@ -700,19 +700,23 @@ void buf_Recycle(cm_buf_t *bp)
         i = BUF_HASH(&bp->fid, &bp->offset);
         lbpp = &(cm_data.buf_scacheHashTablepp[i]);
         for(tbp = *lbpp; tbp; lbpp = &tbp->hashp, tbp = *lbpp) {
-            if (tbp == bp) break;
+            if (tbp == bp) 
+                break;
         }
 
         /* we better find it */
         osi_assertx(tbp != NULL, "buf_Recycle: hash table screwup");
 
         *lbpp = bp->hashp;	/* hash out */
+        bp->hashp = NULL;
 
         /* Remove from file hash */
 
         i = BUF_FILEHASH(&bp->fid);
         prevBp = bp->fileHashBackp;
+        bp->fileHashBackp = NULL;
         nextBp = bp->fileHashp;
+        bp->fileHashp = NULL;
         if (prevBp)
             prevBp->fileHashp = nextBp;
         else
@@ -1550,10 +1554,9 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
 }       
 
 /* Must be called with scp->mx held */
-long buf_ForceDataVersion(cm_scache_t * scp, afs_uint32 fromVersion, afs_uint32 toVersion)
+long buf_ForceDataVersion(cm_scache_t * scp, afs_uint64 fromVersion, afs_uint64 toVersion)
 {
     cm_buf_t * bp;
-    cm_buf_t * nbp;
     unsigned int i;
     int found = 0;
 
@@ -1561,7 +1564,7 @@ long buf_ForceDataVersion(cm_scache_t * scp, afs_uint32 fromVersion, afs_uint32
 
     i = BUF_FILEHASH(&scp->fid);
 
-    lock_ObtainWrite(&buf_globalLock);
+    lock_ObtainRead(&buf_globalLock);
 
     for (bp = cm_data.buf_fileHashTablepp[i]; bp; bp = bp->fileHashp) {
         if (cm_FidCmp(&bp->fid, &scp->fid) == 0) {
@@ -1571,7 +1574,7 @@ long buf_ForceDataVersion(cm_scache_t * scp, afs_uint32 fromVersion, afs_uint32
             }
         }
     }
-    lock_ReleaseWrite(&buf_globalLock);
+    lock_ReleaseRead(&buf_globalLock);
 
     if (found)
         return 0;
@@ -1694,7 +1697,7 @@ int cm_DumpBufHashTable(FILE *outputFile, char *cookie, int lock)
         {
 	    StringCbPrintfA(output, sizeof(output), 
 			    "%s bp=0x%08X, hash=%d, fid (cell=%d, volume=%d, "
-			    "vnode=%d, unique=%d), offset=%x:%08x, dv=%d, "
+			    "vnode=%d, unique=%d), offset=%x:%08x, dv=%I64d, "
 			    "flags=0x%x, cmFlags=0x%x, refCount=%d\r\n",
 			     cookie, (void *)bp, i, bp->fid.cell, bp->fid.volume, 
 			     bp->fid.vnode, bp->fid.unique, bp->offset.HighPart, 
@@ -1712,7 +1715,7 @@ int cm_DumpBufHashTable(FILE *outputFile, char *cookie, int lock)
     for(bp = cm_data.buf_freeListEndp; bp; bp=(cm_buf_t *) osi_QPrev(&bp->q)) {
 	StringCbPrintfA(output, sizeof(output), 
 			 "%s bp=0x%08X, fid (cell=%d, volume=%d, "
-			 "vnode=%d, unique=%d), offset=%x:%08x, dv=%d, "
+			 "vnode=%d, unique=%d), offset=%x:%08x, dv=%I64d, "
 			 "flags=0x%x, cmFlags=0x%x, refCount=%d\r\n",
 			 cookie, (void *)bp, bp->fid.cell, bp->fid.volume, 
 			 bp->fid.vnode, bp->fid.unique, bp->offset.HighPart, 
@@ -1728,7 +1731,7 @@ int cm_DumpBufHashTable(FILE *outputFile, char *cookie, int lock)
     for(bp = cm_data.buf_dirtyListEndp; bp; bp=(cm_buf_t *) osi_QPrev(&bp->q)) {
 	StringCbPrintfA(output, sizeof(output), 
 			 "%s bp=0x%08X, fid (cell=%d, volume=%d, "
-			 "vnode=%d, unique=%d), offset=%x:%08x, dv=%d, "
+			 "vnode=%d, unique=%d), offset=%x:%08x, dv=%I64d, "
 			 "flags=0x%x, cmFlags=0x%x, refCount=%d\r\n",
 			 cookie, (void *)bp, bp->fid.cell, bp->fid.volume, 
 			 bp->fid.vnode, bp->fid.unique, bp->offset.HighPart, 
