@@ -40,6 +40,24 @@ typedef struct cm_SSetPref {
         struct cm_SPref servers[1];/* we overrun this array intentionally...*/
 } cm_SSetPref_t;
 
+#define CM_IOCTLCACHEPARMS		16
+typedef struct cm_cacheParms {
+        afs_uint64 parms[CM_IOCTLCACHEPARMS];
+} cm_cacheParms_t;
+
+
+#define MAXNUMSYSNAMES    16      /* max that current constants allow */
+#define   MAXSYSNAME      128     /* max sysname (i.e. @sys) size */
+extern char *         cm_sysName;
+extern unsigned int   cm_sysNameCount;
+extern char *         cm_sysNameList[MAXNUMSYSNAMES];
+
+/* flags for rxstats pioctl */
+
+#define AFSCALL_RXSTATS_MASK    0x7     /* Valid flag bits */
+#define AFSCALL_RXSTATS_ENABLE  0x1     /* Enable RX stats */
+#define AFSCALL_RXSTATS_DISABLE 0x2     /* Disable RX stats */
+#define AFSCALL_RXSTATS_CLEAR   0x4     /* Clear RX stats */
 
 #ifndef __CM_IOCTL_INTERFACES_ONLY__
 
@@ -53,6 +71,8 @@ extern long cm_IoctlGetFileCellName(smb_ioctl_t *ioctlp, cm_user_t *userp);
 
 extern long cm_IoctlSetACL(smb_ioctl_t *ioctlp, cm_user_t *userp);
 
+extern long cm_IoctlFlushAllVolumes(smb_ioctl_t *ioctlp, cm_user_t *userp);
+
 extern long cm_IoctlFlushVolume(smb_ioctl_t *ioctlp, cm_user_t *userp);
 
 extern long cm_IoctlFlushFile(smb_ioctl_t *ioctlp, cm_user_t *userp);
@@ -61,6 +81,10 @@ extern long cm_IoctlSetVolumeStatus(smb_ioctl_t *ioctlp, cm_user_t *userp);
 
 extern long cm_IoctlGetVolumeStatus(smb_ioctl_t *ioctlp, cm_user_t *userp);
 
+extern long cm_IoctlGetFid(smb_ioctl_t *ioctlp, cm_user_t *userp);
+
+extern long cm_IoctlGetOwner(smb_ioctl_t *ioctlp, cm_user_t *userp);
+
 extern long cm_IoctlWhereIs(smb_ioctl_t *ioctlp, cm_user_t *userp);
 
 extern long cm_IoctlStatMountPoint(smb_ioctl_t *ioctlp, cm_user_t *userp);
@@ -97,8 +121,14 @@ extern long cm_IoctlStoreBehind(smb_ioctl_t *ioctlp, cm_user_t *userp);
 
 extern long cm_IoctlCreateMountPoint(smb_ioctl_t *ioctlp, cm_user_t *userp);
 
+extern long cm_CleanFile(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp);
+
 extern long cm_FlushFile(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp);
 
+extern long cm_FlushVolume(cm_user_t *, cm_req_t *reqp, afs_uint32 cell, afs_uint32 volume);
+
+extern long cm_FlushParent(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp);
+
 extern long cm_IoctlTraceControl(smb_ioctl_t *ioctlp, cm_user_t *userp);
 
 extern long cm_IoctlSetToken(smb_ioctl_t *ioctlp, cm_user_t *userp);
@@ -132,6 +162,17 @@ extern long cm_IoctlFreemountAddCell(smb_ioctl_t *ioctlp, cm_user_t *userp);
 extern long cm_IoctlFreemountRemoveCell(smb_ioctl_t *ioctlp, cm_user_t *userp);
 
 extern long cm_IoctlMemoryDump(smb_ioctl_t *ioctlp, cm_user_t *userp);
+
+extern long cm_IoctlRxStatProcess(smb_ioctl_t *ioctlp, cm_user_t *userp);
+
+extern long cm_IoctlRxStatPeer(smb_ioctl_t *ioctlp, cm_user_t *userp);
+
+extern long cm_IoctlUUIDControl(struct smb_ioctl * ioctlp, struct cm_user *userp);
+
+extern long cm_IoctlPathAvailability(struct smb_ioctl * ioctlp, struct cm_user *userp);
+
+extern long cm_IoctlVolStatTest(struct smb_ioctl *ioctlp, struct cm_user *userp);
+
 #endif /* __CM_IOCTL_INTERFACES_ONLY__ */
 
 #endif /*  __CM_IOCTL_H_ENV__ */
