@@ -16,10 +16,10 @@
 #include <malloc.h>
 #include <string.h>
 
+#include "afsd.h"
 #include <osi.h>
 #include <rx/rx.h>
 
-#include "afsd.h"
 
 osi_rwlock_t cm_userLock;
 
@@ -106,7 +106,7 @@ void cm_ReleaseUser(cm_user_t *userp)
         return;
 
     lock_ObtainWrite(&cm_userLock);
-    osi_assert(userp->refCount-- > 0);
+    osi_assertx(userp->refCount-- > 0, "cm_user_t refCount 0");
     if (userp->refCount == 0) {
         lock_FinalizeMutex(&userp->mx);
         for (ucp = userp->cellInfop; ucp; ucp = ncp) {
@@ -137,7 +137,7 @@ void cm_HoldUserVCRef(cm_user_t *userp)
 void cm_ReleaseUserVCRef(cm_user_t *userp)
 {
     lock_ObtainMutex(&userp->mx);
-    osi_assert(userp->vcRefs-- > 0);
+    osi_assertx(userp->vcRefs-- > 0, "cm_user_t refCount 0");
     lock_ReleaseMutex(&userp->mx);
 }       
 
@@ -150,7 +150,7 @@ void cm_ReleaseUserVCRef(cm_user_t *userp)
  * This routine is called by the cm_Daemon() ie. the periodic daemon.
  * every cm_daemonTokenCheckInterval seconds 
  */
-void cm_CheckTokenCache(long now)
+void cm_CheckTokenCache(time_t now)
 {
     extern smb_vc_t *smb_allVCsp; /* global vcp list */
     smb_vc_t   *vcp;
@@ -196,3 +196,20 @@ void cm_CheckTokenCache(long now)
     }
     lock_ReleaseWrite(&smb_rctLock);
 }
+
+#ifdef USE_ROOT_TOKENS
+/*
+ * Service/Parameters/RootTokens/<cellname>/
+ * -> UseLSA
+ * -> Keytab (required if UseLSA is 0)
+ * -> Principal (required if there is more than one principal in the keytab)
+ * -> Realm (required if realm is not upper-case of <cellname>
+ * -> RequireEncryption 
+ */
+
+void
+cm_RefreshRootTokens(void)
+{
+
+}
+#endif 
