@@ -77,7 +77,7 @@ cm_ShutdownVolume(void)
             cm_VolumeStatusNotification(volp, volp->ro.ID, volp->ro.state, vl_alldown);
         if (volp->bk.ID)
             cm_VolumeStatusNotification(volp, volp->bk.ID, volp->bk.state, vl_alldown);
-
+        volp->cbExpiresRO = 0;
         lock_FinalizeMutex(&volp->mx);
     }
 
@@ -118,6 +118,7 @@ void cm_InitVolume(int newFile, long maxVols)
                     cm_VolumeStatusNotification(volp, volp->ro.ID, vl_alldown, volp->ro.state);
                 if (volp->bk.ID)
                     cm_VolumeStatusNotification(volp, volp->bk.ID, vl_alldown, volp->bk.state);
+                volp->cbExpiresRO = 0;
             }
         }
         osi_EndOnce(&once);
@@ -768,6 +769,7 @@ long cm_GetVolumeByName(struct cm_cell *cellp, char *volumeNamep,
         volp->rw.state = volp->ro.state = volp->bk.state = vl_unknown;
         volp->rw.nextp = volp->ro.nextp = volp->bk.nextp = NULL;
         volp->rw.flags = volp->ro.flags = volp->bk.flags = 0;
+        volp->cbExpiresRO = 0;
         cm_AddVolumeToNameHashTable(volp);
         lock_ReleaseWrite(&cm_volumeLock);
     }
@@ -1033,6 +1035,10 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
             cm_VolumeStatusNotification(volp, volp->rw.ID, volp->rw.state, vl_online);
             volp->rw.state = vl_online;
             online = 1;
+        } else if (code == CM_ERROR_NOACCESS) {
+            cm_VolumeStatusNotification(volp, volp->rw.ID, volp->rw.state, vl_unknown);
+            volp->rw.state = vl_unknown;
+            online = 1;
         }
     }
 
@@ -1062,6 +1068,10 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
             cm_VolumeStatusNotification(volp, volp->ro.ID, volp->ro.state, vl_online);
             volp->ro.state = vl_online;
             online = 1;
+        } else if (code == CM_ERROR_NOACCESS) {
+            cm_VolumeStatusNotification(volp, volp->ro.ID, volp->ro.state, vl_unknown);
+            volp->ro.state = vl_unknown;
+            online = 1;
         }
     }
 
@@ -1091,6 +1101,10 @@ cm_CheckOfflineVolume(cm_volume_t *volp, afs_uint32 volID)
             cm_VolumeStatusNotification(volp, volp->bk.ID, volp->bk.state, vl_online);
             volp->bk.state = vl_online;
             online = 1;
+        } else if (code == CM_ERROR_NOACCESS) {
+            cm_VolumeStatusNotification(volp, volp->bk.ID, volp->bk.state, vl_unknown);
+            volp->bk.state = vl_unknown;
+            online = 1;
         }
     }
 
@@ -1117,24 +1131,16 @@ void cm_CheckOfflineVolumes(void)
     lock_ReleaseWrite(&cm_volumeLock);
 }
 
-void
-cm_UpdateVolumeStatus(cm_volume_t *volp, afs_uint32 volID)
+
+static void
+cm_UpdateVolumeStatusInt(cm_volume_t *volp, struct cm_vol_state *statep)
 {
-    struct cm_vol_state * statep = NULL;
     enum volstatus newStatus;
     cm_serverRef_t *tsrp;
     cm_server_t *tsp;
     int someBusy = 0, someOffline = 0, allOffline = 1, allBusy = 1, allDown = 1;
 
-    if (volp->rw.ID == volID) {
-        statep = &volp->rw;
-    } else if (volp->ro.ID == volID) {
-        statep = &volp->ro;
-    } else if (volp->bk.ID == volID) {
-        statep = &volp->bk;
-    }
-
-    if (!statep) {
+    if (!volp || !statep) {
 #ifdef DEBUG
         DebugBreak();
 #endif
@@ -1171,13 +1177,39 @@ cm_UpdateVolumeStatus(cm_volume_t *volp, afs_uint32 volID)
     else
 	newStatus = vl_online;
 
-
     if (statep->ID && statep->state != newStatus)
         cm_VolumeStatusNotification(volp, statep->ID, statep->state, newStatus);
 
     statep->state = newStatus;
 }
 
+void
+cm_UpdateVolumeStatus(cm_volume_t *volp, afs_uint32 volID)
+{
+
+    if (volp->rw.ID == volID) {
+        cm_UpdateVolumeStatusInt(volp, &volp->rw);
+    } else if (volp->ro.ID == volID) {
+        cm_UpdateVolumeStatusInt(volp, &volp->ro);
+    } else if (volp->bk.ID == volID) {
+        cm_UpdateVolumeStatusInt(volp, &volp->bk);
+    } else {
+        /*
+         * If we are called with volID == 0 then something has gone wrong.
+         * Most likely a race occurred in the server volume list maintenance.
+         * Since we don't know which volume's status should be updated, 
+         * just update all of them that are known to exist.  Better to be 
+         * correct than fast.
+         */
+        if (volp->rw.ID != 0)
+            cm_UpdateVolumeStatusInt(volp, &volp->rw);
+        if (volp->ro.ID != 0)
+            cm_UpdateVolumeStatusInt(volp, &volp->ro);
+        if (volp->bk.ID != 0)
+            cm_UpdateVolumeStatusInt(volp, &volp->bk);
+    }
+}
+
 /*
 ** Finds all volumes that reside on this server and reorders their
 ** RO list according to the changed rank of server.
@@ -1498,3 +1530,38 @@ enum volstatus cm_GetVolumeStatus(cm_volume_t *volp, afs_uint32 volID)
     }
 }
 
+/* Renew .readonly volume callbacks that are more than
+ * 30 minutes old.  (A volume callback is issued for 2 hours.)
+ */
+void 
+cm_VolumeRenewROCallbacks(void)
+{
+    cm_volume_t * volp;
+    time_t minexp = time(NULL) + 90 * 60;
+
+    lock_ObtainRead(&cm_volumeLock);
+    for (volp = cm_data.allVolumesp; volp; volp=volp->allNextp) {
+        if ( volp->cbExpiresRO > 0 && volp->cbExpiresRO < minexp) {
+            cm_req_t      req;
+            cm_fid_t      fid;
+            cm_scache_t * scp;
+
+            fid.cell = volp->cellp->cellID;
+            fid.volume = volp->ro.ID;
+            fid.vnode = 1;
+            fid.unique = 1;
+
+            cm_InitReq(&req);
+
+            lock_ReleaseRead(&cm_volumeLock);
+            if (cm_GetSCache(&fid, &scp, cm_rootUserp, &req) == 0) {
+                lock_ObtainMutex(&scp->mx);
+                cm_GetCallback(scp, cm_rootUserp, &req, 1);
+                lock_ReleaseMutex(&scp->mx);
+                cm_ReleaseSCache(scp);
+            }
+            lock_ObtainRead(&cm_volumeLock);
+        }
+    }
+    lock_ReleaseRead(&cm_volumeLock);
+}
