@@ -22,6 +22,7 @@
 
 #include <osi.h>
 #include <afsint.h>
+#include <WINNT\afsreg.h>
 
 #include "fs_utils.h"
 #include "cmd.h"
@@ -87,10 +88,31 @@ IsFreelanceRoot(char *apath)
 
     code = pioctl(apath, VIOC_FILE_CELL_NAME, &blob, 1);
     if (code == 0)
-        return !strcmp("Freelance.Local.Root",space);
+        return !stricmp("Freelance.Local.Root",space);
     return 1;   /* assume it is because it is more restrictive that way */
 }
 
+static const char * NetbiosName(void)
+{
+    static char buffer[1024] = "AFS";
+    HKEY  parmKey;
+    DWORD code;
+    DWORD dummyLen;
+    DWORD enabled = 0;
+
+    code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
+                         0, KEY_QUERY_VALUE, &parmKey);
+    if (code == ERROR_SUCCESS) {
+        dummyLen = sizeof(buffer);
+        code = RegQueryValueEx(parmKey, "NetbiosName", NULL, NULL,
+			       buffer, &dummyLen);
+        RegCloseKey (parmKey);
+    } else {
+	strcpy(buffer, "AFS");
+    }
+    return buffer;
+}
+
 #define AFSCLIENT_ADMIN_GROUPNAME "AFS Client Admins"
 
 static BOOL IsAdmin (void)
@@ -247,8 +269,8 @@ char *apath; {
 }
 
 
-static ListLinkCmd(as)
-register struct cmd_syndesc *as; {
+static ListLinkCmd(register struct cmd_syndesc *as, void *arock)
+{
     register afs_int32 code;
     struct ViceIoctl blob;
     int error;
@@ -338,6 +360,21 @@ register struct cmd_syndesc *as; {
 	    strncpy(parent_dir, true_name, last_component - true_name + 1);
 	    parent_dir[last_component - true_name + 1] = 0;
 	    last_component++;   /*Skip the slash*/
+
+#ifdef WIN32
+	    if (!InAFS(parent_dir)) {
+		const char * nbname = NetbiosName();
+		int len = (int)strlen(nbname);
+
+		if (parent_dir[0] == '\\' && parent_dir[1] == '\\' &&
+		    parent_dir[len+2] == '\\' &&
+		    parent_dir[len+3] == '\0' &&
+		    !strnicmp(nbname,&parent_dir[2],len))
+		{
+		    sprintf(parent_dir,"\\\\%s\\all\\", nbname);
+		}
+	    }
+#endif
 	}
 	else {
 	    /*
@@ -357,7 +394,7 @@ register struct cmd_syndesc *as; {
 	    continue;
 	}
 	blob.in = last_component;
-	blob.in_size = strlen(last_component)+1;
+	blob.in_size = (long)strlen(last_component)+1;
 	blob.out_size = MAXSIZE;
 	blob.out = space;
 	memset(space, 0, MAXSIZE);
@@ -383,41 +420,56 @@ register struct cmd_syndesc *as; {
     return error;
 }
 
-static MakeLinkCmd(as)
-register struct cmd_syndesc *as; {
+static MakeLinkCmd(register struct cmd_syndesc *as, void *arock)
+{
     register afs_int32 code;
     struct ViceIoctl blob;
     char * parent;
+    char path[1024] = "";
 
-    parent = Parent(as->parms[0].items->data);
+    strcpy(path, as->parms[0].items->data);
+    parent = Parent(path);
 
     if (!InAFS(parent)) {
-	fprintf(stderr,"%s: symlinks must be created within the AFS file system\n", pn);
-	return 1;
+#ifdef WIN32
+	const char * nbname = NetbiosName();
+	int len = (int)strlen(nbname);
+
+	if (parent[0] == '\\' && parent[1] == '\\' &&
+	    parent[len+2] == '\\' &&
+	    parent[len+3] == '\0' &&
+	    !strnicmp(nbname,&parent[2],len))
+	{
+	    sprintf(path,"%sall\\%s", parent, &as->parms[0].items->data[strlen(parent)]);
+	    parent = Parent(path);
+	    if (!InAFS(parent)) {
+		fprintf(stderr,"%s: symlinks must be created within the AFS file system\n", pn);
+		return 1;
+	    }
+	} else 
+#endif
+	{
+	    fprintf(stderr,"%s: symlinks must be created within the AFS file system\n", pn);
+	    return 1;
+	}
     }
 
+#ifdef WIN32
     if ( IsFreelanceRoot(parent) && !IsAdmin() ) {
 	fprintf(stderr,"%s: Only AFS Client Administrators may alter the root.afs volume\n", pn);
 	return 1;
     }
 
-    strcpy(space, as->parms[1].items->data);
-#ifdef WIN32
     /* create symlink with a special pioctl for Windows NT, since it doesn't
      * have a symlink system call.
      */
-
-    /* TODO: Code needs to go here to prevent the creation of symlinks
-     * in \\AFS\all when not in the "AFS Client Admins" group.
-     */
-
     blob.out_size = 0;
-    blob.in_size = 1 + strlen(space);
-    blob.in = space;
+    blob.in_size = 1 + (long)strlen(as->parms[1].items->data);
+    blob.in = as->parms[1].items->data;
     blob.out = NULL;
-    code = pioctl(as->parms[0].items->data, VIOC_SYMLINK, &blob, 0);
+    code = pioctl(path, VIOC_SYMLINK, &blob, 0);
 #else /* not WIN32 */
-    code = symlink(space, as->parms[0].items->data);
+    code = symlink(as->parms[1].items->data, path);
 #endif /* not WIN32 */
     if (code) {
 	Die(errno, as->parms[0].items->data);
@@ -432,8 +484,8 @@ register struct cmd_syndesc *as; {
  *	    symlink (or ``.'' if none is provided)
  *      tp: Set to point to the actual name of the symlink to nuke.
  */
-static RemoveLinkCmd(as)
-register struct cmd_syndesc *as; {
+static RemoveLinkCmd(register struct cmd_syndesc *as, void *arock)
+{
     register afs_int32 code=0;
     struct ViceIoctl blob;
     register struct cmd_item *ti;
@@ -447,9 +499,24 @@ register struct cmd_syndesc *as; {
 	if (!tp)
 	    tp = (char *) strrchr(ti->data, '/');
 	if (tp) {
-	    strncpy(tbuffer, ti->data, code=tp-ti->data+1);  /* the dir name */
+	    strncpy(tbuffer, ti->data, code=(afs_int32)(tp-ti->data+1));  /* the dir name */
             tbuffer[code] = 0;
 	    tp++;   /* skip the slash */
+
+#ifdef WIN32
+	    if (!InAFS(tbuffer)) {
+		const char * nbname = NetbiosName();
+		int len = (int)strlen(nbname);
+
+		if (tbuffer[0] == '\\' && tbuffer[1] == '\\' &&
+		     tbuffer[len+2] == '\\' &&
+		     tbuffer[len+3] == '\0' &&
+		     !strnicmp(nbname,&tbuffer[2],len))
+		{
+		    sprintf(tbuffer,"\\\\%s\\all\\", nbname);
+		}
+	    }
+#endif
 	}
 	else {
 	    fs_ExtractDriveLetter(ti->data, tbuffer);
@@ -458,7 +525,7 @@ register struct cmd_syndesc *as; {
             fs_StripDriveLetter(tp, tp, 0);
 	}
 	blob.in = tp;
-	blob.in_size = strlen(tp)+1;
+	blob.in_size = (int)strlen(tp)+1;
 	blob.out = lsbuffer;
 	blob.out_size = sizeof(lsbuffer);
 	code = pioctl(tbuffer, VIOC_LISTSYMLINK, &blob, 0);
@@ -471,7 +538,7 @@ register struct cmd_syndesc *as; {
 	    continue;	/* don't bother trying */
 	}
 
-        if ( IsFreelanceRoot(Parent(tp)) && !IsAdmin() ) {
+        if ( IsFreelanceRoot(tbuffer) && !IsAdmin() ) {
             fprintf(stderr,"symlink: Only AFS Client Administrators may alter the root.afs volume\n");
             code = 1;
             continue;   /* skip */
@@ -479,7 +546,7 @@ register struct cmd_syndesc *as; {
 
 	blob.out_size = 0;
 	blob.in = tp;
-	blob.in_size = strlen(tp)+1;
+	blob.in_size = (long)strlen(tp)+1;
 	code = pioctl(tbuffer, VIOC_DELSYMLINK, &blob, 0);
 	if (code) {
 	    Die(errno, ti->data);
@@ -522,15 +589,16 @@ char **argv; {
 
     osi_Init();
 
-    ts = cmd_CreateSyntax("list", ListLinkCmd, 0, "list symlink");    
+    ts = cmd_CreateSyntax("list", ListLinkCmd, NULL, "list symlink");    
     cmd_AddParm(ts, "-name", CMD_LIST, 0, "name");
     
-    ts = cmd_CreateSyntax("make", MakeLinkCmd, 0, "make symlink");
+    ts = cmd_CreateSyntax("make", MakeLinkCmd, NULL, "make symlink");
     cmd_AddParm(ts, "-name", CMD_SINGLE, 0, "name");
     cmd_AddParm(ts, "-to", CMD_SINGLE, 0, "target");
 
-    ts = cmd_CreateSyntax("rm", RemoveLinkCmd, 0, "remove symlink");
+    ts = cmd_CreateSyntax("remove", RemoveLinkCmd, NULL, "remove symlink");
     cmd_AddParm(ts, "-name", CMD_LIST, 0, "name");
+    cmd_CreateAlias(ts, "rm");
 
     code = cmd_Dispatch(argc, argv);
 
@@ -552,8 +620,12 @@ void Die(code, filename)
 	else fprintf(stderr,"%s: Invalid argument.\n", pn);
     }
     else if (code == ENOENT) {
-	if (filename) fprintf(stderr,"%s: File '%s' doesn't exist\n", pn, filename);
-	else fprintf(stderr,"%s: no such file returned\n", pn);
+	if (filename) fprintf(stderr,"%s: File '%s' doesn't exist.\n", pn, filename);
+	else fprintf(stderr,"%s: no such file returned.\n", pn);
+    }
+    else if (code == EEXIST) {
+	if (filename) fprintf(stderr,"%s: File '%s' already exists.\n", pn, filename);
+	else fprintf(stderr,"%s: the specified file already exists.\n", pn);
     }
     else if (code == EROFS)  fprintf(stderr,"%s: You can not change a backup or readonly volume\n", pn);
     else if (code == EACCES || code == EPERM) {
@@ -584,7 +656,7 @@ void Die(code, filename)
 #ifdef WIN32
 	fprintf(stderr, ": code 0x%x\n", code);
 #else /* not WIN32 */
-	fprintf(stderr,": %s\n", error_message(code));
+	fprintf(stderr,": %s\n", afs_error_message(code));
 #endif /* not WIN32 */
     }
 } /*Die*/
