@@ -14,13 +14,16 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/SOLARIS/osi_vfsops.c,v 1.18.14.1 2006/06/30 14:07:31 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/SOLARIS/osi_vfsops.c,v 1.18.14.4 2007/11/01 16:57:37 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* statistics stuff */
 #include "h/modctl.h"
 #include "h/syscall.h"
+#if defined(AFS_SUN511_ENV)
+#include <sys/vfs_opreg.h>
+#endif
 #include <sys/kobj.h>
 
 
@@ -43,15 +46,19 @@ afs_mount(struct vfs *afsp, struct vnode *amvp, struct mounta *uap,
 
     AFS_STATCNT(afs_mount);
 
-    if (!suser(credp)) {
+#if defined(AFS_SUN510_ENV)
+    if (secpolicy_fs_mount(credp, amvp, afsp) != 0) {
+#else
+    if (!afs_osi_suser(credp)) {
+#endif
 	AFS_GUNLOCK();
-	return EPERM;
+	return (EPERM);
     }
     afsp->vfs_fstype = afsfstype;
 
     if (afs_globalVFS) {	/* Don't allow remounts. */
 	AFS_GUNLOCK();
-	return EBUSY;
+	return (EBUSY);
     }
 
     afs_globalVFS = afsp;
@@ -75,9 +82,13 @@ afs_unmount(struct vfs *afsp, struct AFS_UCRED *credp)
     AFS_GLOCK();
     AFS_STATCNT(afs_unmount);
 
-    if (!suser(credp)) {
-	AFS_GUNLOCK();
-	return EPERM;
+#if defined(AFS_SUN510_ENV)
+    if (secpolicy_fs_unmount(credp, afsp) != 0) {
+#else
+    if (!afs_osi_suser(credp)) {
+#endif
+        AFS_GUNLOCK();
+        return (EPERM);
     }
     afs_globalVFS = 0;
     afs_shutdown();
@@ -237,16 +248,32 @@ afs_swapvp(struct vfs *afsp, struct vnode **avpp, char *nm)
 }
 
 
-#ifdef AFS_SUN510_ENV
-struct fs_operation_def afs_vfsops_template[] = {
-    { VFSNAME_MOUNT,		afs_mount },
-    { VFSNAME_UNMOUNT,		afs_unmount },
-    { VFSNAME_ROOT,		afs_root },
-    { VFSNAME_STATVFS,		afs_statvfs },
-    { VFSNAME_SYNC,		afs_sync },
-    { VFSNAME_VGET,		afs_vget },
-    { VFSNAME_MOUNTROOT,	afs_mountroot },
-    { VFSNAME_FREEVFS,		fs_freevfs },
+#if defined(AFS_SUN511_ENV)
+/* The following list must always be NULL-terminated */
+static const fs_operation_def_t afs_vfsops_template[] = {
+    VFSNAME_MOUNT,		{ .vfs_mount = afs_mount },
+    VFSNAME_UNMOUNT,		{ .vfs_unmount = afs_unmount },
+    VFSNAME_ROOT,		{ .vfs_root = afs_root },
+    VFSNAME_STATVFS,		{ .vfs_statvfs = afs_statvfs },
+    VFSNAME_SYNC,		{ .vfs_sync = afs_sync },
+    VFSNAME_VGET,		{ .vfs_vget = afs_vget },
+    VFSNAME_MOUNTROOT,  	{ .vfs_mountroot = afs_mountroot },
+    VFSNAME_FREEVFS,		{ .vfs_freevfs = fs_freevfs },
+    NULL,			NULL
+};
+struct vfsops *afs_vfsopsp;
+#elif defined(AFS_SUN510_ENV)
+/* The following list must always be NULL-terminated */
+const fs_operation_def_t afs_vfsops_template[] = {
+    VFSNAME_MOUNT,		afs_mount,
+    VFSNAME_UNMOUNT,		afs_unmount,
+    VFSNAME_ROOT,		afs_root,
+    VFSNAME_STATVFS,		afs_statvfs,
+    VFSNAME_SYNC,		afs_sync,
+    VFSNAME_VGET,		afs_vget,
+    VFSNAME_MOUNTROOT,  	afs_mountroot,
+    VFSNAME_FREEVFS,		fs_freevfs,
+    NULL,			NULL
 };
 struct vfsops *afs_vfsopsp;
 #else
