@@ -1,5 +1,5 @@
 /* 
- * $Id: aklog_main.c,v 1.12.2.9 2007/11/05 22:10:48 shadow Exp $
+ * $Id: aklog_main.c,v 1.12.2.12 2008/01/05 04:46:52 shadow Exp $
  *
  * Copyright 1990,1991 by the Massachusetts Institute of Technology
  * For distribution and copying rights, see the file "mit-copyright.h"
@@ -36,7 +36,7 @@
 
 #if !defined(lint) && !defined(SABER)
 static char *rcsid =
-	"$Id: aklog_main.c,v 1.12.2.9 2007/11/05 22:10:48 shadow Exp $";
+	"$Id: aklog_main.c,v 1.12.2.12 2008/01/05 04:46:52 shadow Exp $";
 #endif /* lint || SABER */
 
 #include <afsconfig.h>
@@ -670,6 +670,33 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
 	status = get_credv5(context, name, primary_instance, realm_of_cell,
 			    &v5cred);
 
+#if !defined(USING_HEIMDAL) && defined(HAVE_KRB5_DECODE_TICKET)
+	if (status == 0 && strcmp(realm_of_cell, "") == 0) {
+	    krb5_error_code code;
+	    krb5_ticket *ticket;
+
+	    code = krb5_decode_ticket(&v5cred->ticket, &ticket);
+
+	    if (code != 0) {
+		fprintf(stderr,
+			"%s: Couldn't decode ticket to determine realm for "
+			"cell %s.\n",
+			progname, cell_to_use);
+	    } else {
+		int len = realm_len(context, ticket->server);
+		/* This really shouldn't happen. */
+		if (len > REALM_SZ-1)
+		    len = REALM_SZ-1;
+
+		strncpy(realm_of_cell, realm_data(context, ticket->server), 
+			len);
+		realm_of_cell[len] = 0;
+
+		krb5_free_ticket(context, ticket);
+	    }
+	}
+#endif
+
 	if ((status == KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN || status == KRB5KRB_ERR_GENERIC) &&
 	    !realm_of_cell[0]) {
 	    char *afs_realm = afs_realm_of_cell(context, &ak_cellconfig, TRUE);
@@ -720,7 +747,7 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
 	    }
 	    fprintf(stderr, "%s: Couldn't get %s AFS tickets:\n",
 		    progname, cell_to_use);
-		com_err(progname, status, "while getting AFS tickets");
+		afs_com_err(progname, status, "while getting AFS tickets");
 	    return(AKLOG_KERBEROS);
 	}
 
@@ -774,7 +801,7 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
 	    status = krb5_524_convert_creds(context, v5cred, &cred);
 
 	    if (status) {
-		com_err(progname, status, "while converting tickets "
+		afs_com_err(progname, status, "while converting tickets "
 			"to Kerberos V4 format");
 		return(AKLOG_KERBEROS);
 	    }
@@ -832,7 +859,7 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
 	    if ((status = get_user_realm(context, realm_of_user))) {
 		fprintf(stderr, "%s: Couldn't determine realm of user:)",
 			progname);
-		com_err(progname, status, " while getting realm");
+		afs_com_err(progname, status, " while getting realm");
 		return(AKLOG_KERBEROS);
 	    }
 	    if (strcmp(realm_of_user, realm_of_cell)) {
@@ -939,11 +966,14 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
 	    printf("Setting tokens. %s / %s @ %s \n",
 		    aclient.name, aclient.instance, aclient.cell );
 	}
+#ifndef AFS_AIX51_ENV
 	/* on AIX 4.1.4 with AFS 3.4a+ if a write is not done before 
 	 * this routine, it will not add the token. It is not clear what 
-	 * is going on here! So we will do the following operation
+	 * is going on here! So we will do the following operation.
+	 * On AIX 5, it causes the parent program to die, so we won't.
 	 */
 	write(2,"",0); /* dummy write */
+#endif
 #ifndef WINDOWS
 	if ((status = ktc_SetToken(&aserver, &atoken, &aclient, afssetpag))) {
 	    fprintf(stderr, 
@@ -1744,9 +1774,9 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
     code = krb5_kt_resolve(context, keytab, &kt);
     if (code) {
         if (keytab)
-            com_err(progname, code, "while resolving keytab %s", keytab);
+            afs_com_err(progname, code, "while resolving keytab %s", keytab);
         else
-            com_err(progname, code, "while resolving default keytab");
+            afs_com_err(progname, code, "while resolving default keytab");
         goto cleanup;
     }
 
@@ -1765,7 +1795,7 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
 	    }
         }
         if (code) {
-	    com_err(progname, code,"while scanning keytab entries");
+	    afs_com_err(progname, code,"while scanning keytab entries");
 	    goto cleanup;
         }
     } else {
@@ -1773,7 +1803,7 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
         int best = -1;
         memset(new, 0, sizeof *new);
         if ((code == krb5_kt_start_seq_get(context, kt, cursor))) {
-            com_err(progname, code, "while starting keytab scan");
+            afs_com_err(progname, code, "while starting keytab scan");
             goto cleanup;
         }
         while (!(code = krb5_kt_next_entry(context, kt, new, cursor))) {
@@ -1789,12 +1819,12 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
             } else krb5_free_keytab_entry_contents(context, new);
         }
         if ((i = krb5_kt_end_seq_get(context, kt, cursor))) {
-            com_err(progname, i, "while ending keytab scan");
+            afs_com_err(progname, i, "while ending keytab scan");
             code = i;
             goto cleanup;
         }
         if (best < 0) {
-            com_err(progname, code, "while scanning keytab");
+            afs_com_err(progname, code, "while scanning keytab");
             goto cleanup;
         }
         deref_keyblock_enctype(session_key) = deref_entry_enctype(entry);
@@ -1805,7 +1835,7 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
 #if USING_HEIMDAL
     if ((code = krb5_generate_random_keyblock(context,
 					      deref_keyblock_enctype(session_key), session_key))) {
-        com_err(progname, code, "while making session key");
+        afs_com_err(progname, code, "while making session key");
         goto cleanup;
     }
     enc_tkt_reply->flags.initial = 1;
@@ -1828,7 +1858,7 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
 #else
     if ((code = krb5_c_make_random_key(context,
 				       deref_keyblock_enctype(session_key), session_key))) {
-        com_err(progname, code, "while making session key");
+        afs_com_err(progname, code, "while making session key");
         goto cleanup;
     }
     enc_tkt_reply->magic = KV5M_ENC_TKT_PART;
@@ -1891,12 +1921,12 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
         ASN1_MALLOC_ENCODE(EncTicketPart, buf, buf_size,
 			   enc_tkt_reply, &buf_len, code);
         if(code) {
-            com_err(progname, code, "while encoding ticket");
+            afs_com_err(progname, code, "while encoding ticket");
             goto cleanup;
         }
 
         if(buf_len != buf_size) {
-            com_err(progname, code,
+            afs_com_err(progname, code,
 		    "%d != %d while encoding ticket (internal ASN.1 encoder error",
 		    buf_len, buf_size);
             goto cleanup;
@@ -1914,7 +1944,7 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
         if (buf) free(buf);
         if (crypto) krb5_crypto_destroy(context, crypto);
         if(code) {
-            com_err(progname, code, "while %s", what);
+            afs_com_err(progname, code, "while %s", what);
             goto cleanup;
         }
     } /* crypto block */
@@ -1926,7 +1956,7 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
     ticket_reply->server = service_principal;
     ticket_reply->enc_part2 = enc_tkt_reply;
     if ((code = krb5_encrypt_tkt_part(context, &deref_entry_keyblock(entry), ticket_reply))) {
-        com_err(progname, code, "while making ticket");
+        afs_com_err(progname, code, "while making ticket");
         goto cleanup;
     }
     ticket_reply->enc_part.kvno = entry->vno;
@@ -1936,17 +1966,17 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
 
     if ((code = krb5_copy_principal(context, service_principal,
 				    &creds->server))) {
-        com_err(progname, code, "while copying service principal");
+        afs_com_err(progname, code, "while copying service principal");
         goto cleanup;
     }
     if ((code = krb5_copy_principal(context, client_principal,
 				    &creds->client))) {
-        com_err(progname, code, "while copying client principal");
+        afs_com_err(progname, code, "while copying client principal");
         goto cleanup;
     }
     if ((code = krb5_copy_keyblock_contents(context, session_key,
 					    &deref_session_key(creds)))) {
-        com_err(progname, code, "while copying session key");
+        afs_com_err(progname, code, "while copying session key");
         goto cleanup;
     }
 
@@ -1964,7 +1994,7 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
         ;
     else if ((code = krb5_copy_addresses(context,
 					 deref_enc_tkt_addrs(enc_tkt_reply), &creds->addresses))) {
-        com_err(progname, code, "while copying addresses");
+        afs_com_err(progname, code, "while copying addresses");
         goto cleanup;
     }
 
@@ -1974,13 +2004,13 @@ static krb5_error_code get_credv5_akimpersonate(krb5_context context,
 	ASN1_MALLOC_ENCODE(Ticket, creds->ticket.data, creds->ticket.length,
 			   ticket_reply, &creds_tkt_len, code);
 	if(code) {
-	    com_err(progname, code, "while encoding ticket");
+	    afs_com_err(progname, code, "while encoding ticket");
 	    goto cleanup;
 	}
     }
 #else
     if ((code = encode_krb5_ticket(ticket_reply, &temp))) {
-	com_err(progname, code, "while encoding ticket");
+	afs_com_err(progname, code, "while encoding ticket");
 	goto cleanup;
     }
     creds->ticket = *temp;
