@@ -15,7 +15,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_packet.c,v 1.62.2.6 2007/01/12 03:49:56 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_packet.c,v 1.62.2.10 2007/10/30 15:16:45 shadow Exp $");
 
 #ifdef KERNEL
 #if defined(UKERNEL)
@@ -88,13 +88,7 @@ RCSID
 #include "rx_globals.h"
 #include <lwp.h>
 #include <assert.h>
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -2226,25 +2220,23 @@ rxi_SendPacket(struct rx_call *call, struct rx_connection *conn,
 	    p->retryTime = p->timeSent;	/* resend it very soon */
 	    clock_Addmsec(&(p->retryTime),
 			  10 + (((afs_uint32) p->backoff) << 8));
-
+	    /* Some systems are nice and tell us right away that we cannot
+	     * reach this recipient by returning an error code. 
+	     * So, when this happens let's "down" the host NOW so
+	     * we don't sit around waiting for this host to timeout later.
+	     */
+	    if (call && 
 #ifdef AFS_NT40_ENV
-	    /* Windows is nice -- it can tell us right away that we cannot
-	     * reach this recipient by returning an WSAEHOSTUNREACH error
-	     * code.  So, when this happens let's "down" the host NOW so
-	     * we don't sit around waiting for this host to timeout later.
-	     */
-		if (call && code == -1 && errno == WSAEHOSTUNREACH)
-			call->lastReceiveTime = 0;
+		code == -1 && WSAGetLastError() == WSAEHOSTUNREACH
+#elif defined(AFS_LINUX20_ENV) && defined(KERNEL)
+		code == -ENETUNREACH
+#elif defined(AFS_DARWIN_ENV) && defined(KERNEL)
+		code == EHOSTUNREACH
+#else
+		0
 #endif
-#if defined(KERNEL) && defined(AFS_LINUX20_ENV)
-	    /* Linux is nice -- it can tell us right away that we cannot
-	     * reach this recipient by returning an ENETUNREACH error
-	     * code.  So, when this happens let's "down" the host NOW so
-	     * we don't sit around waiting for this host to timeout later.
-	     */
-	    if (call && code == -ENETUNREACH)
+		)
 		call->lastReceiveTime = 0;
-#endif
 	}
 #ifdef KERNEL
 #ifdef RX_KERNEL_TRACE
@@ -2420,24 +2412,23 @@ rxi_SendPacketList(struct rx_call *call, struct rx_connection *conn,
 		clock_Addmsec(&(p->retryTime),
 			      10 + (((afs_uint32) p->backoff) << 8));
 	    }
+	    /* Some systems are nice and tell us right away that we cannot
+	     * reach this recipient by returning an error code. 
+	     * So, when this happens let's "down" the host NOW so
+	     * we don't sit around waiting for this host to timeout later.
+	     */
+	    if (call && 
 #ifdef AFS_NT40_ENV
-	    /* Windows is nice -- it can tell us right away that we cannot
-	     * reach this recipient by returning an WSAEHOSTUNREACH error
-	     * code.  So, when this happens let's "down" the host NOW so
-	     * we don't sit around waiting for this host to timeout later.
-	     */
-	    if (call && code == -1 && errno == WSAEHOSTUNREACH)
-		call->lastReceiveTime = 0;
+		code == -1 && WSAGetLastError() == WSAEHOSTUNREACH
+#elif defined(AFS_LINUX20_ENV) && defined(KERNEL)
+		code == -ENETUNREACH
+#elif defined(AFS_DARWIN_ENV) && defined(KERNEL)
+		code == EHOSTUNREACH
+#else
+		0
 #endif
-#if defined(KERNEL) && defined(AFS_LINUX20_ENV)
-	    /* Linux is nice -- it can tell us right away that we cannot
-	     * reach this recipient by returning an ENETUNREACH error
-	     * code.  So, when this happens let's "down" the host NOW so
-	     * we don't sit around waiting for this host to timeout later.
-	     */
-	    if (call && code == -ENETUNREACH)
+		)
 		call->lastReceiveTime = 0;
-#endif
 	}
 #if	defined(AFS_SUN5_ENV) && defined(KERNEL)
 	if (!istack && waslocked)
