@@ -14,6 +14,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/rxkad/rxkad_server.c,v 1.21.2.4 2007/10/30 15:16:46 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -55,7 +57,7 @@ static struct rx_securityOps rxkad_server_ops = {
     rxkad_CheckPacket,		/* check data packet */
     rxkad_DestroyConnection,
     rxkad_GetStats,
-    rxkad_SetConfiguration,
+    0,				/* spare 1 */
     0,				/* spare 2 */
     0,				/* spare 3 */
 };
@@ -89,7 +91,7 @@ init_random_int32(void)
 
     gettimeofday(&key, NULL);
     LOCK_RM;
-    fc_keysched(&key, random_int32_schedule);
+    fc_keysched((struct ktc_encryptionKey*)&key, random_int32_schedule);
     UNLOCK_RM;
 }
 
@@ -325,9 +327,7 @@ rxkad_CheckResponse(struct rx_securityClass *aobj,
 	code =
 	    tkt_DecodeTicket5(tix, tlen, tsp->get_key, tsp->get_key_rock,
 			      kvno, client.name, client.instance, client.cell,
-			      &sessionkey, &host, &start, &end, 
-			      tsp->flags & RXS_CONFIG_FLAGS_DISABLE_DOTCHECK,
-			      tsp->alt_decrypt);
+			      &sessionkey, &host, &start, &end);
 	if (code)
 	    return code;
     }
@@ -452,36 +452,3 @@ rxkad_GetServerInfo(struct rx_connection * aconn, rxkad_level * level,
     } else
 	return RXKADNOAUTH;
 }
-
-/* Set security object configuration variables */
-afs_int32 rxkad_SetConfiguration(struct rx_securityClass *aobj,
-                                 struct rx_connection *aconn, 
-                                 rx_securityConfigVariables atype,
-		                         void * avalue, void **currentValue)
-{
-    struct rxkad_sprivate *private = 
-    (struct rxkad_sprivate *) aobj->privateData;
-
-    switch (atype) {
-    case RXS_CONFIG_FLAGS:
-        if (currentValue) {
-            *((afs_uint32 *)currentValue) = private->flags;
-        } else { 
-            private->flags = (afs_uint32) avalue;
-        }
-        break;
-    default:
-        break;
-    }
-    return 0;
-}
-
-int rxkad_SetAltDecryptProc(struct rx_securityClass *aobj,
-			    rxkad_alt_decrypt_func alt_decrypt)
-{
-    struct rxkad_sprivate *private =
-    (struct rxkad_sprivate *)aobj->privateData;
-
-    private->alt_decrypt = alt_decrypt;
-    return 0;
-}
