@@ -62,7 +62,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/rxkad/ticket5.c,v 1.7 2004/04/02 16:41:27 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/rxkad/ticket5.c,v 1.10.2.2 2007/10/30 15:16:46 shadow Exp $");
 
 #if defined(UKERNEL)
 #include "../afs/sysincludes.h"
@@ -81,13 +81,7 @@ RCSID
 #else
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <des.h>
@@ -101,6 +95,8 @@ RCSID
 #include "v5gen.h"
 #include "v5der.c"
 #include "v5gen.c"
+#include "md4.h"
+#include "md5.h"
 
 /*
  * Principal conversion Taken from src/lib/krb5/krb/conv_princ from MIT Kerberos.  If you
@@ -240,11 +236,6 @@ tkt_DecodeTicket5(char *ticket, afs_int32 ticket_len,
 	v5_serv_kvno = *t5.enc_part.kvno;
     }
 
-
-    code = (*get_key) (get_key_rock, v5_serv_kvno, &serv_key);
-    if (code)
-	goto unknown_key;
-
     /* Check that the key type really fit into 8 bytes */
     switch (t5.enc_part.etype) {
     case ETYPE_DES_CBC_CRC:
@@ -260,6 +251,10 @@ tkt_DecodeTicket5(char *ticket, afs_int32 ticket_len,
 	|| t5.enc_part.cipher.length % 8 != 0)
 	goto bad_ticket;
 
+    code = (*get_key) (get_key_rock, v5_serv_kvno, &serv_key);
+    if (code)
+	goto unknown_key;
+
     /* Decrypt data here, save in plain, assume it will shrink */
     code =
 	krb5_des_decrypt(&serv_key, t5.enc_part.etype,
@@ -283,7 +278,7 @@ tkt_DecodeTicket5(char *ticket, afs_int32 ticket_len,
 	v5_comp1 = decr_part.cname.name_string.val[1];
 	p = sconv_list;
 	while (p->v4_str) {
-	    if (strncmp(p->v5_str, v5_comp0, p->len) == 0) {
+	    if (strcmp(p->v5_str, v5_comp0) == 0) {
 		/*
 		 * It is, so set the new name now, and chop off
 		 * instance's domain name if requested.
@@ -383,6 +378,40 @@ tkt_DecodeTicket5(char *ticket, afs_int32 ticket_len,
 
 }
 
+static int
+verify_checksum_md4(void *data, size_t len,
+		    void *cksum, size_t cksumsz,
+		    struct ktc_encryptionKey *key)
+{
+    MD4_CTX md4;
+    unsigned char tmp[16];
+
+    MD4_Init(&md4);
+    MD4_Update(&md4, data, len);
+    MD4_Final(tmp, &md4);
+
+    if (memcmp(tmp, cksum, cksumsz) != 0)
+	return 1;
+    return 0;
+}
+
+static int
+verify_checksum_md5(void *data, size_t len,
+		    void *cksum, size_t cksumsz,
+		    struct ktc_encryptionKey *key)
+{
+    MD5_CTX md5;
+    unsigned char tmp[16];
+
+    MD5_Init(&md5);
+    MD5_Update(&md5, data, len);
+    MD5_Final(tmp, &md5);
+
+    if (memcmp(tmp, cksum, cksumsz) != 0)
+	return 1;
+    return 0;
+}
+
 static int
 verify_checksum_crc(void *data, size_t len, void *cksum, size_t cksumsz,
 		    struct ktc_encryptionKey *key)
@@ -413,7 +442,7 @@ krb5_des_decrypt(struct ktc_encryptionKey *key, int etype, void *in,
     des_key_schedule s;
     char cksum[24];
     size_t cksumsz;
-    int ret = 1;	/* failure */
+    int ret = 1;		/* failure */
 
     cksum_func = NULL;
 
@@ -430,12 +459,12 @@ krb5_des_decrypt(struct ktc_encryptionKey *key, int etype, void *in,
     case ETYPE_DES_CBC_MD4:
 	memset(&ivec, 0, sizeof(ivec));
 	cksumsz = 16;
-	/* FIXME: cksum_func = verify_checksum_md4 */ ;
+	cksum_func = verify_checksum_md4;
 	break;
     case ETYPE_DES_CBC_MD5:
 	memset(&ivec, 0, sizeof(ivec));
 	cksumsz = 16;
-	/* FIXME: cksum_func = verify_checksum_md5 */ ;
+	cksum_func = verify_checksum_md5;
 	break;
     default:
 	abort();
