@@ -10,7 +10,8 @@
 /* Get CM initialization parameters. */
 #include <afsconfig.h>
 
-RCSID("$Header: /cvs/openafs/src/venus/test/getinitparams.c,v 1.3 2001/07/05 15:21:12 shadow Exp $");
+RCSID
+    ("$Header: /cvs/openafs/src/venus/test/getinitparams.c,v 1.5.14.1 2007/10/31 04:09:42 shadow Exp $");
 
 #include <afs/param.h>
 #include <stdio.h>
@@ -32,7 +33,8 @@ RCSID("$Header: /cvs/openafs/src/venus/test/getinitparams.c,v 1.3 2001/07/05 15:
 
 
 
-int GetInitParamsCmd(struct cmd_syndesc *as)
+int
+GetInitParamsCmd(struct cmd_syndesc *as, void *arock)
 {
     struct cm_initparams cm_initParams;
     struct ViceIoctl blob;
@@ -42,13 +44,13 @@ int GetInitParamsCmd(struct cmd_syndesc *as)
     int fd;
 
     if (as->parms[0].items) {
-        file = as->parms[0].items->data;
+	file = as->parms[0].items->data;
     }
 
     if (file) {
 	printf("ioctl test\n");
 	fd = open(file, O_RDONLY, 0);
-	if (fd<0) {
+	if (fd < 0) {
 	    perror("open");
 	    exit(1);
 	}
@@ -56,14 +58,14 @@ int GetInitParamsCmd(struct cmd_syndesc *as)
 	printf("lpioctl test\n");
     }
 
-    blob.in = (char*)0;
+    blob.in = (char *)0;
     blob.in_size = 0;
     blob.out = (char *)&cm_initParams;
     blob.out_size = sizeof(struct cm_initparams);
 
     if (file) {
 	code = ioctl(fd, VIOC_GETINITPARAMS, &blob);
-	if (code<0) {
+	if (code < 0) {
 	    perror("ioctl: Error getting CM initialization parameters");
 	    exit(1);
 	}
@@ -92,12 +94,12 @@ int GetInitParamsCmd(struct cmd_syndesc *as)
 
 
 main(ac, av)
-int ac;
-char **av;
+     int ac;
+     char **av;
 {
     int code;
     struct cmd_syndesc *ts;
-    
+
 #ifdef	AFS_AIX32_ENV
     /*
      * The following signal action for AIX is necessary so that in case of a 
@@ -106,18 +108,17 @@ char **av;
      * generated which, in many cases, isn't too useful.
      */
     struct sigaction nsa;
-    
+
     sigemptyset(&nsa.sa_mask);
     nsa.sa_handler = SIG_DFL;
     nsa.sa_flags = SA_FULLDUMP;
     sigaction(SIGSEGV, &nsa, NULL);
 #endif
 
-    ts = cmd_CreateSyntax((char *)0, GetInitParamsCmd, (char *)0,
+    ts = cmd_CreateSyntax(NULL, GetInitParamsCmd, NULL,
 			  "Get CM initialization parameters");
-    
+
     cmd_AddParm(ts, "-file", CMD_SINGLE, CMD_OPTIONAL, "filename in AFS");
     code = cmd_Dispatch(ac, av);
     exit(code);
 }
-
