@@ -18,18 +18,13 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/xstat/xstat_fs.c,v 1.10 2006/03/06 03:11:31 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/xstat/xstat_fs.c,v 1.10.2.2 2007/10/30 15:16:59 shadow Exp $");
 
 #include "xstat_fs.h"		/*Interface for this module */
 #include <lwp.h>		/*Lightweight process package */
 
-#ifdef HAVE_STRING_H
+#include <afs/afsutil.h>
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #define LWP_STACK_SIZE	(16 * 1024)
 
